@@ -1,14 +1,25 @@
+#include "config.h"
+
+#include <locale.h>
 
 #include <glib.h>
+#include <unistd.h>
+#include <stdlib.h>
 #include <libgnomevfs/gnome-vfs.h>
 
-#include "totem-pl-parser.c"
+#include "totem-pl-parser.h"
+#include "totem-pl-parser-mini.h"
+
+#define USE_DATA
 
 static GMainLoop *loop = NULL;
-static GList *list = NULL;
-static gboolean option_recurse = TRUE;
+static gboolean option_no_recurse = FALSE;
 static gboolean option_debug = FALSE;
 static gboolean option_data = FALSE;
+static gboolean option_force = FALSE;
+static gboolean option_disable_unsafe = FALSE;
+static char *option_base_uri = NULL;
+static char **files = NULL;
 
 static void
 header (const char *message)
@@ -18,6 +29,7 @@ header (const char *message)
 	g_print ("\n");
 }
 
+#if 0
 static void
 test_relative_real (const char *url, const char *output)
 {
@@ -57,13 +69,24 @@ test_relative (void)
 	test_relative_real ("http://foobar.com/test.avi",
 			"/home/hadess/test/file.m3u");
 }
+#endif
 
 static void
-entry_added (TotemPlParser *parser, const char *uri, const char *title,
-		const char *genre, gpointer data)
+entry_metadata_foreach (const char *key,
+			const char *value,
+			gpointer data)
 {
-	g_print ("added URI '%s' with title '%s' genre '%s'\n", uri,
-			title ? title : "empty", genre);
+	if (g_ascii_strcasecmp (key, "url") == 0)
+		return;
+	g_print ("\t%s = '%s'\n", key, value);
+}
+
+static void
+entry_parsed (TotemPlParser *parser, const char *uri,
+	      GHashTable *metadata, gpointer data)
+{
+	g_print ("added URI '%s'\n", uri);
+	g_hash_table_foreach (metadata, (GHFunc) entry_metadata_foreach, NULL);
 }
 
 static void
@@ -71,7 +94,7 @@ test_parsing_real (TotemPlParser *pl, const char *url)
 {
 	TotemPlParserResult res;
 
-	res = totem_pl_parser_parse (pl, url, FALSE);
+	res = totem_pl_parser_parse_with_base (pl, url, option_base_uri, FALSE);
 	if (res != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 		switch (res) {
 		case TOTEM_PL_PARSER_RESULT_UNHANDLED:
@@ -95,13 +118,12 @@ push_parser (gpointer data)
 {
 	TotemPlParser *pl = (TotemPlParser *)data;
 
-	if (list != NULL) {
-		GList *l;
+	if (files != NULL) {
+		guint i;
 
-		for (l = list; l != NULL; l = l->next) {
-			test_parsing_real (pl, l->data);
+		for (i = 0; files[i] != NULL; ++i) {
+			test_parsing_real (pl, files[i]);
 		}
-		g_list_free (list);
 	} else {
 		//test_parsing_real (pl, "file:///mnt/cdrom");
 		test_parsing_real (pl, "file:///home/hadess/Movies");
@@ -116,6 +138,8 @@ push_parser (gpointer data)
 	return FALSE;
 }
 
+#ifdef USE_DATA
+
 #define READ_CHUNK_SIZE 8192
 #define MIME_READ_CHUNK_SIZE 1024
 
@@ -181,27 +205,36 @@ test_data_get_data (const char *uri, guint *len)
 
 	return buffer;
 }
+#endif /* USE_DATA */
 
 static void
 test_data (void)
 {
-	GList *l;
-	guint len;
+	guint i;
 
-	for (l = list; l != NULL; l = l->next) {
+	for (i = 0; files[i] != NULL; ++i) {
+		char *filename = files[i];
+		gboolean retval;
+#ifdef USE_DATA
 		char *data;
+		guint len;
 
-		data = test_data_get_data (l->data, &len);
+		data = test_data_get_data (filename, &len);
 		if (data == NULL) {
-			g_message ("Couldn't get data for %s", (char *) l->data);
+			g_message ("Couldn't get data for %s", filename);
 			continue;
 		}
-		if (totem_pl_parser_can_parse_from_data (data, len, TRUE)) {
-			g_message ("IS a playlist: %s", (char *) l->data);
-		} else {
-			g_message ("ISNOT playlist: %s", (char *) l->data);
-		}
+		retval = totem_pl_parser_can_parse_from_data (data, len, TRUE);
 		g_free (data);
+#else
+		retval = totem_pl_parser_can_parse_from_filename (filename, TRUE);
+#endif /* USE_DATA */
+
+		if (retval != FALSE) {
+			g_message ("IS a playlist: %s", filename);
+		} else {
+			g_message ("ISNOT playlist: %s", filename);
+		}
 	}
 }
 
@@ -222,9 +255,12 @@ test_parsing (void)
 {
 	TotemPlParser *pl = totem_pl_parser_new ();
 
-	g_object_set (G_OBJECT (pl), "recurse", option_recurse, NULL);
-	g_object_set (G_OBJECT (pl), "debug", option_debug, NULL);
-	g_signal_connect (G_OBJECT (pl), "entry", G_CALLBACK (entry_added), NULL);
+	g_object_set (pl, "recurse", !option_no_recurse,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "entry-parsed", G_CALLBACK (entry_parsed), NULL);
 	g_signal_connect (G_OBJECT (pl), "playlist-start", G_CALLBACK (playlist_started), NULL);
 	g_signal_connect (G_OBJECT (pl), "playlist-end", G_CALLBACK (playlist_ended), NULL);
 
@@ -236,53 +272,58 @@ test_parsing (void)
 
 int main (int argc, char **argv)
 {
-	gnome_vfs_init();
+	GOptionEntry option_entries [] =
+	{
+		{ "no-recurse", 'n', 0, G_OPTION_ARG_NONE, &option_no_recurse, "Disable recursion", NULL },
+		{ "debug", 'd', 0, G_OPTION_ARG_NONE, &option_debug, "Enable debug", NULL },
+		{ "data", 't', 0, G_OPTION_ARG_NONE, &option_data, "Use data instead of filename", NULL },
+		{ "force", 'f', 0, G_OPTION_ARG_NONE, &option_force, "Force parsing", NULL },
+		{ "disable-unsafe", 'u', 0, G_OPTION_ARG_NONE, &option_disable_unsafe, "Disabling unsafe playlist-types", NULL },
+		{ "base-uri", 'b', 0, G_OPTION_ARG_STRING, &option_base_uri, "Base URI to resolve relative items from", NULL },
+		{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &files, NULL, "[URI...]" },
+		{ NULL }
+	};
+	GOptionContext *context;
+	GError *error = NULL;
+	gboolean retval;
 
-	while (argc > 1 && argv[1]) {
-		if (strcmp (argv[1], "--no-recurse") == 0 || strcmp (argv[1], "-n") == 0) {
-			g_print ("Disabling recursion\n");
-			option_recurse = FALSE;
-			argv++;
-			argc--;
-		} else if (strcmp (argv[1], "--debug") == 0 || strcmp (argv[1], "-d") == 0) {
-			g_print ("Enabling debug\n");
-			option_debug = TRUE;
-			argv++;
-			argc--;
-		} else if (strcmp (argv[1], "--help") == 0 || strcmp (argv[1], "-h") == 0) {
-			g_print ("Usage: %s <-n | --no-recurse> <-d | --debug> <-h | --help> <-d | --data > <url>\n", argv[0]);
-			return 0;
-		} else if (strcmp (argv[1], "--data") == 0 || strcmp (argv[1], "-t") == 0) {
-			g_print ("Using data, instead of filenames\n");
-			option_data = TRUE;
-			argv++;
-			argc--;
-		} else /* other options here */ {
-			break;
-		}
+	setlocale (LC_ALL, "");
+
+	g_thread_init (NULL);
+
+	context = g_option_context_new (NULL);
+	g_option_context_add_main_entries (context, option_entries, NULL);
+
+	retval = g_option_context_parse (context, &argc, &argv, &error);
+	g_option_context_free (context);
+
+	if (!retval) {
+		g_print ("Error parsing arguments: %s\n", error->message);
+		g_error_free (error);
+
+		g_print ("Usage: %s <-n | --no-recurse> <-d | --debug> <-h | --help> <-t | --data > <-u | --disable-unsafe> <url>\n", argv[0]);
+		exit (1);
 	}
 
-	if (argc < 2 && option_data != FALSE) {
+	gnome_vfs_init();
+
+	if (option_data != FALSE && files == NULL) {
 		g_message ("Please pass specific files to check by data");
 		return 1;
 	}
 
-	if (argc == 1) {
+	if (files == NULL) {
+#if 0
 		test_relative ();
+#endif
 		test_parsing ();
 	} else {
-		int i;
-		for (i = 1; i < argc; i++) {
-			list = g_list_prepend (list, argv[i]);
-		}
-		list = g_list_reverse (list);
-		if (option_data == FALSE) {
-			test_parsing ();
-		} else {
+		if (option_data) {
 			test_data ();
+		} else {
+			test_parsing ();
 		}
 	}
 
 	return 0;
 }
-
