@@ -24,7 +24,7 @@
  * permission are above and beyond the permissions granted by the GPL license
  * Totem is covered by.
  *
- * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * Monday 7th February 2005: Christian Schaller: Add exception clause.
  * See license_change file for details.
  *
  */
@@ -154,6 +154,7 @@ totem_interface_get_full_path (const char *name)
 {
 	char *filename;
 
+#ifdef TOTEM_RUN_IN_SOURCE_TREE
 	/* Try the glade file in the source tree first */
 	filename = g_build_filename ("..", "data", name, NULL);
 	if (g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE)
@@ -169,6 +170,10 @@ totem_interface_get_full_path (const char *name)
 			return NULL;
 		}
 	}
+#else
+	filename = g_build_filename (DATADIR,
+	                                "totem", name, NULL);
+#endif
 
 	return filename;
 }
@@ -186,11 +191,12 @@ totem_gtk_plug_get_toplevel (GtkPlug *plug)
 
 	do
 	{
+		/* FIXME: multi-head */
 		if (XQueryTree (GDK_DISPLAY (), xid, &root,
 					&parent, &children, &nchildren) == 0)
 		{
 			g_warning ("Couldn't find window manager window");
-			return None;
+			return NULL;
 		}
 
 		if (root == parent) {
@@ -215,7 +221,7 @@ totem_interface_set_transient_for (GtkWindow *window, GtkWindow *parent)
 		if (toplevel != NULL) {
 			gdk_window_set_transient_for
 				(GTK_WIDGET (window)->window, toplevel);
-			gdk_window_unref (toplevel);
+			g_object_unref (toplevel);
 		}
 	} else {
 		gtk_window_set_transient_for (GTK_WINDOW (window),
@@ -223,3 +229,97 @@ totem_interface_set_transient_for (GtkWindow *window, GtkWindow *parent)
 	}
 }
 
+char *
+totem_interface_get_license (void)
+{
+	const char *license[] = {
+		N_("Totem is free software; you can redistribute it and/or modify "
+		   "it under the terms of the GNU General Public License as published by "
+		   "the Free Software Foundation; either version 2 of the License, or "
+		   "(at your option) any later version."),
+		N_("Totem is distributed in the hope that it will be useful, "
+		   "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+		   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
+		   "GNU General Public License for more details."),
+		N_("You should have received a copy of the GNU General Public License "
+		   "along with Totem; if not, write to the Free Software Foundation, Inc., "
+		   "59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"),
+		N_("Totem contains an exception to allow the use of proprietary "
+		   "GStreamer plugins.")
+	};
+	return g_strjoin ("\n\n",
+			  _(license[0]),
+			  _(license[1]),
+			  _(license[2]),
+			  _(license[3]),
+			  NULL);
+}
+
+void
+totem_interface_boldify_label (GladeXML *xml, const char *name)
+{
+	GtkWidget *widget;
+
+	widget = glade_xml_get_widget (xml, name);
+
+	if (widget == NULL) {
+		g_warning ("widget '%s' not found", name);
+		return;
+	}
+
+	/* this way is probably better, but for some reason doesn't work with
+	 * labels with mnemonics.
+
+	static PangoAttrList *pattrlist = NULL;
+
+	if (pattrlist == NULL) {
+		PangoAttribute *attr;
+
+		pattrlist = pango_attr_list_new ();
+		attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
+		attr->start_index = 0;
+		attr->end_index = G_MAXINT;
+		pango_attr_list_insert (pattrlist, attr);
+	}
+	gtk_label_set_attributes (GTK_LABEL (widget), pattrlist);*/
+
+	gchar *str_final;
+	str_final = g_strdup_printf ("<b>%s</b>", gtk_label_get_label (GTK_LABEL (widget)));
+	gtk_label_set_markup_with_mnemonic (GTK_LABEL (widget), str_final);
+	g_free (str_final);
+}
+
+void
+totem_interface_italicise_label (GladeXML *xml, const char *name)
+{
+	GtkWidget *widget;
+
+	widget = glade_xml_get_widget (xml, name);
+
+	if (widget == NULL) {
+		g_warning ("widget '%s' not found", name);
+		return;
+	}
+
+	/* this way is probably better, but for some reason doesn't work with
+	 * labels with mnemonics.
+
+	static PangoAttrList *pattrlist = NULL;
+
+	if (pattrlist == NULL) {
+		PangoAttribute *attr;
+
+		pattrlist = pango_attr_list_new ();
+		attr = pango_attr_style_new (PANGO_STYLE_ITALIC);
+		attr->start_index = 0;
+		attr->end_index = G_MAXINT;
+		pango_attr_list_insert (pattrlist, attr);
+	}
+	gtk_label_set_attributes (GTK_LABEL (widget), pattrlist);*/
+
+	gchar *str_final;
+	str_final = g_strdup_printf ("<i>%s</i>", gtk_label_get_label (GTK_LABEL (widget)));
+	gtk_label_set_markup_with_mnemonic (GTK_LABEL (widget), str_final);
+	g_free (str_final);
+}
+
