@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
- * Copyright (C) 2001-2006 Bastien Nocera <hadess@hadess.net>
+ * Copyright (C) 2001-2007 Bastien Nocera <hadess@hadess.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * permission are above and beyond the permissions granted by the GPL license
  * Totem is covered by.
  *
- * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * Monday 7th February 2005: Christian Schaller: Add exception clause.
  * See license_change file for details.
  *
  */
@@ -35,7 +35,6 @@
 
 #ifndef HAVE_GTK_ONLY
 #include <gnome.h>
-#include "totem-gromit.h"
 #endif /* !HAVE_GTK_ONLY */
 
 #include <string.h>
@@ -50,7 +49,6 @@
 #endif
 
 #include "bacon-video-widget.h"
-#include "bacon-video-widget-properties.h"
 #include "totem-statusbar.h"
 #include "totem-time-label.h"
 #include "totem-session.h"
@@ -100,7 +98,6 @@ static void update_fullscreen_size (Totem *totem);
 static gboolean popup_hide (Totem *totem);
 static void update_buttons (Totem *totem);
 static void update_media_menu_items (Totem *totem);
-static void update_seekable (Totem *totem, gboolean force_false);
 static void playlist_changed_cb (GtkWidget *playlist, Totem *totem);
 static void play_pause_set_label (Totem *totem, TotemStates state);
 static gboolean on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, Totem *totem);
@@ -156,6 +153,9 @@ totem_action_save_state (Totem *totem)
 	char *contents, *filename;
 	const char *page_id;
 
+	if (totem->win == NULL)
+		return;
+
 	keyfile = g_key_file_new ();
 	g_key_file_set_integer (keyfile, "State",
 			"window_w", totem->window_w);
@@ -174,7 +174,7 @@ totem_action_save_state (Totem *totem)
 
 	contents = g_key_file_to_data (keyfile, NULL, NULL);
 	g_key_file_free (keyfile);
-	filename = g_build_filename (g_get_home_dir (), ".gnome2", "totem", NULL);
+	filename = g_build_filename (totem_dot_dir (), "state.ini", NULL);
 	g_file_set_contents (filename, contents, -1, NULL);
 
 	g_free (filename);
@@ -195,12 +195,6 @@ totem_action_exit (Totem *totem)
 	if (totem == NULL)
 		exit (0);
 
-#ifdef HAVE_REMOTE
-	if (totem->remote != NULL) {
-		g_object_unref (G_OBJECT (totem->remote));
-	}
-#endif /* HAVE_REMOTE */
-
 	if (totem->win != NULL) {
 		gtk_widget_hide (totem->win);
 		display = gtk_widget_get_display (totem->win);
@@ -210,9 +204,7 @@ totem_action_exit (Totem *totem)
 		gtk_widget_hide (totem->prefs);
 	}
 
-#ifndef HAVE_GTK_ONLY
-	totem_gromit_clear (TRUE);
-#endif /* !HAVE_GTK_ONLY */
+	totem_object_plugins_shutdown ();
 
 	if (display != NULL)
 		gdk_display_sync (display);
@@ -240,6 +232,8 @@ totem_action_exit (Totem *totem)
 	if (totem->win)
 		gtk_widget_destroy (GTK_WIDGET (totem->win));
 
+	g_object_unref (totem);
+
 	gnome_vfs_shutdown ();
 
 	exit (0);
@@ -282,12 +276,14 @@ play_pause_set_label (Totem *totem, TotemStates state)
 				_("Playing"));
 		id = GTK_STOCK_MEDIA_PAUSE;
 		tip = N_("Pause");
+		totem_playlist_set_playing (totem->playlist, TOTEM_PLAYLIST_STATUS_PLAYING);
 		break;
 	case STATE_PAUSED:
 		totem_statusbar_set_text (TOTEM_STATUSBAR (totem->statusbar),
 				_("Paused"));
 		id = GTK_STOCK_MEDIA_PLAY;
 		tip = N_("Play");
+		totem_playlist_set_playing (totem->playlist, TOTEM_PLAYLIST_STATUS_PAUSED);
 		break;
 	case STATE_STOPPED:
 		totem_statusbar_set_text (TOTEM_STATUSBAR (totem->statusbar),
@@ -295,9 +291,11 @@ play_pause_set_label (Totem *totem, TotemStates state)
 		totem_statusbar_set_time_and_length
 			(TOTEM_STATUSBAR (totem->statusbar), 0, 0);
 		id = GTK_STOCK_MEDIA_PLAY;
+		totem_playlist_set_playing (totem->playlist, TOTEM_PLAYLIST_STATUS_NONE);
 		tip = N_("Play");
 		break;
 	default:
+		g_assert_not_reached ();
 		return;
 	}
 
@@ -313,6 +311,8 @@ play_pause_set_label (Totem *totem, TotemStates state)
 	}
 
 	totem->state = state;
+
+	g_object_notify (G_OBJECT (totem), "playing");
 }
 
 void
@@ -346,7 +346,7 @@ totem_action_eject (Totem *totem)
 void
 totem_action_show_properties (Totem *totem)
 {
-        if (totem_is_fullscreen (totem) == FALSE)
+	if (totem_is_fullscreen (totem) == FALSE)
 		totem_sidebar_set_current_page (totem, "properties");
 }
 
@@ -355,6 +355,7 @@ totem_action_play (Totem *totem)
 {
 	GError *err = NULL;
 	int retval;
+	char *msg, *disp;
 
 	if (totem->mrl == NULL)
 		return;
@@ -364,23 +365,18 @@ totem_action_play (Totem *totem)
 
 	retval = bacon_video_widget_play (totem->bvw,  &err);
 	play_pause_set_label (totem, retval ? STATE_PLAYING : STATE_STOPPED);
-	if (totem_is_fullscreen (totem) != FALSE)
-		totem_scrsaver_set_state (totem->scr, !retval);
 
-	if (retval == FALSE)
-	{
-		char *msg, *disp;
+	if (retval != FALSE)
+		return;
 
-		disp = totem_uri_escape_for_display (totem->mrl);
-		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
-		g_free (disp);
+	disp = totem_uri_escape_for_display (totem->mrl);
+	msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
+	g_free (disp);
 
-		totem_playlist_set_playing (totem->playlist, FALSE);
-		totem_action_error (msg, err->message, totem);
-		totem_action_stop (totem);
-		g_free (msg);
-		g_error_free (err);
-	}
+	totem_action_error (msg, err->message, totem);
+	totem_action_stop (totem);
+	g_free (msg);
+	g_error_free (err);
 }
 
 static void
@@ -404,7 +400,6 @@ totem_action_seek (Totem *totem, double pos)
 		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 		g_free (disp);
 
-		totem_playlist_set_playing (totem->playlist, FALSE);
 		totem_action_error (msg, err->message, totem);
 		totem_action_stop (totem);
 		g_free (msg);
@@ -454,7 +449,7 @@ totem_action_open_dialog (Totem *totem, const char *path, gboolean play)
 }
 
 static gboolean
-totem_action_load_media (Totem *totem, MediaType type)
+totem_action_load_media (Totem *totem, MediaType type, const char *device)
 {
 	char **mrls;
 	char *msg;
@@ -468,7 +463,7 @@ totem_action_load_media (Totem *totem, MediaType type)
 		return FALSE;
 	}
 
-	mrls = bacon_video_widget_get_mrls (totem->bvw, type);
+	mrls = bacon_video_widget_get_mrls (totem->bvw, type, device);
 	if (mrls == NULL)
 	{
 		msg = g_strdup_printf (_("Totem could not play this media (%s) although a plugin is present to handle it."), _(totem_cd_get_human_readable_name (type)));
@@ -513,9 +508,7 @@ totem_action_load_media_device (Totem *totem, const char *device)
 		case MEDIA_TYPE_DVD:
 		case MEDIA_TYPE_VCD:
 		case MEDIA_TYPE_CDDA:
-			bacon_video_widget_set_media_device
-				(BACON_VIDEO_WIDGET (totem->bvw), device_path);
-			retval = totem_action_load_media (totem, type);
+			retval = totem_action_load_media (totem, type, device_path);
 			if (retval == FALSE) {
 				totem_action_set_mrl_and_play (totem, NULL);
 			}
@@ -543,11 +536,11 @@ totem_action_play_media_device (Totem *totem, const char *device)
 }
 
 void
-totem_action_play_media (Totem *totem, MediaType type)
+totem_action_play_media (Totem *totem, MediaType type, const char *device)
 {
 	char *mrl;
 
-	if (totem_action_load_media (totem, type) != FALSE) {
+	if (totem_action_load_media (totem, type, device) != FALSE) {
 		mrl = totem_playlist_get_current_mrl (totem->playlist);
 		totem_action_set_mrl_and_play (totem, mrl);
 		g_free (mrl);
@@ -558,7 +551,7 @@ void
 totem_action_stop (Totem *totem)
 {
 	bacon_video_widget_stop (totem->bvw);
-	totem_scrsaver_enable (totem->scr);
+	play_pause_set_label (totem, STATE_STOPPED);
 }
 
 void
@@ -585,12 +578,9 @@ totem_action_play_pause (Totem *totem)
 	{
 		bacon_video_widget_play (totem->bvw, NULL);
 		play_pause_set_label (totem, STATE_PLAYING);
-		if (totem_is_fullscreen (totem) != FALSE)
-			totem_scrsaver_disable (totem->scr);
 	} else {
 		bacon_video_widget_pause (totem->bvw);
 		play_pause_set_label (totem, STATE_PAUSED);
-		totem_scrsaver_enable (totem->scr);
 	}
 }
 
@@ -601,7 +591,6 @@ totem_action_pause (Totem *totem)
 	{
 		bacon_video_widget_pause (totem->bvw);
 		play_pause_set_label (totem, STATE_PAUSED);
-		totem_scrsaver_enable (totem->scr);
 	}
 }
 
@@ -618,7 +607,12 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 {
 	if (event->changed_mask == GDK_WINDOW_STATE_MAXIMIZED) {
 		totem->maximised = (event->new_window_state
-				& GDK_WINDOW_STATE_MAXIMIZED);
+				& GDK_WINDOW_STATE_MAXIMIZED) != 0;
+                gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (totem->statusbar),
+                                                   !totem->maximised);
+		totem_action_set_sensitivity ("zoom-1-2", !totem->maximised);
+		totem_action_set_sensitivity ("zoom-1-1", !totem->maximised);
+		totem_action_set_sensitivity ("zoom-2-1", !totem->maximised);
 		return FALSE;
 	}
 
@@ -632,9 +626,6 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 		bacon_video_widget_set_fullscreen (totem->bvw, TRUE);
 		totem_action_set_cursor (totem, FALSE);
 
-		if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
-			totem_scrsaver_disable (totem->scr);
-
 		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
 		show_controls (totem, FALSE);
 		totem_action_set_sensitivity ("fullscreen", FALSE);
@@ -645,8 +636,6 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 		bacon_video_widget_set_fullscreen (totem->bvw, FALSE);
 		totem_action_set_cursor (totem, TRUE);
 
-		totem_scrsaver_enable (totem->scr);
-
 		action = gtk_action_group_get_action (totem->main_action_group,
 				"show-controls");
 
@@ -659,6 +648,8 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 		totem_action_set_sensitivity ("fullscreen", TRUE);
 	}
 
+	g_object_notify (G_OBJECT (totem), "fullscreen");
+
 	return FALSE;
 }
 
@@ -764,15 +755,15 @@ totem_action_open_location (Totem *totem)
 	if (glade == NULL)
 		return;
 
+	dialog = glade_xml_get_widget (glade, "open_uri_dialog");
+	entry = glade_xml_get_widget (glade, "uri");
+
 	/* Get item from clipboard to fill entry in Open Location... dialog */
 	clipboard_location = totem_open_location_set_from_clipboard (totem);
 	if (clipboard_location != NULL && strcmp (clipboard_location, "") != 0)
-		gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget (glade, "uri")), clipboard_location);
+		gtk_entry_set_text (GTK_ENTRY (entry), clipboard_location);
 	g_free (clipboard_location);
 
-	dialog = glade_xml_get_widget (glade, "open_uri_dialog");
-	entry = glade_xml_get_widget (glade, "uri");
-
 	completion = gtk_entry_completion_new();
 	model = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
 	gtk_entry_set_completion (GTK_ENTRY (entry), completion);
@@ -870,7 +861,7 @@ totem_action_take_screenshot (Totem *totem)
 
 	filename = g_build_filename (DATADIR,
 			"totem", "screenshot.glade", NULL);
-	dialog = totem_screenshot_new (filename, pixbuf);
+	dialog = totem_screenshot_new (pixbuf);
 	g_free (filename);
 
 	gtk_dialog_run (GTK_DIALOG (dialog));
@@ -915,13 +906,6 @@ totem_get_nice_name_for_stream (Totem *totem)
 	return retval;
 }
 
-static void
-update_skip_to (Totem *totem, gint64 time)
-{
-	if (totem->skipto != NULL)
-		totem_skipto_update_range (totem->skipto, time);
-}
-
 static void
 update_mrl_label (Totem *totem, const char *name)
 {
@@ -938,7 +922,7 @@ update_mrl_label (Totem *totem, const char *name)
 		totem_statusbar_set_time_and_length (TOTEM_STATUSBAR
 				(totem->statusbar), 0, time / 1000);
 
-		update_skip_to (totem, time);
+		g_object_notify (G_OBJECT (totem), "stream-length");
 
 		/* Update the mrl label */
 		escaped = g_markup_escape_text (name, strlen (name));
@@ -959,7 +943,7 @@ update_mrl_label (Totem *totem, const char *name)
 		totem_statusbar_set_text (TOTEM_STATUSBAR (totem->statusbar),
 				_("Stopped"));
 
-		update_skip_to (totem, 0);
+		g_object_notify (G_OBJECT (totem), "stream-length");
 
 		/* Update the mrl label */
 		text = g_strdup_printf
@@ -986,22 +970,19 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		g_free (totem->mrl);
 		totem->mrl = NULL;
 		bacon_video_widget_close (totem->bvw);
+		totem_file_closed (totem);
+		play_pause_set_label (totem, TOTEM_PLAYLIST_STATUS_NONE);
 	}
 
-	/* Reset the properties and wait for the signal*/
-	bacon_video_widget_properties_reset
-		(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties));
-
 	if (mrl == NULL)
 	{
 		retval = FALSE;
 
+		play_pause_set_label (totem, TOTEM_PLAYLIST_STATUS_NONE);
+
 		/* Play/Pause */
 		totem_action_set_sensitivity ("play", FALSE);
 
-		/* Seek bar and seek buttons */
-		update_seekable (totem, FALSE);
-
 		/* Volume */
 		totem_main_set_sensitivity ("tcw_volume_button", FALSE);
 		totem_action_set_sensitivity ("volume-up", FALSE);
@@ -1022,8 +1003,6 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 
 		/* Set the logo */
 		bacon_video_widget_set_logo_mode (totem->bvw, TRUE);
-		gtk_widget_set_sensitive (GTK_WIDGET (totem->properties),
-				FALSE);
 		update_mrl_label (totem, NULL);
 	} else {
 		gboolean caps;
@@ -1042,10 +1021,6 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		/* Play/Pause */
 		totem_action_set_sensitivity ("play", TRUE);
 
-		/* Seek bar */
-		update_seekable (totem,
-				bacon_video_widget_is_seekable (totem->bvw));
-
 		/* Volume */
 		caps = bacon_video_widget_can_set_volume (totem->bvw);
 		totem_main_set_sensitivity ("tcw_volume_button", caps);
@@ -1060,11 +1035,8 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		/* Clear the playlist */
 		totem_action_set_sensitivity ("clear-playlist", retval);
 
-		gtk_widget_set_sensitive
-			(GTK_WIDGET (totem->properties), retval);
-
 		/* Set the playlist */
-		totem_playlist_set_playing (totem->playlist, retval);
+		play_pause_set_label (totem, retval ? STATE_PAUSED : STATE_STOPPED);
 
 		if (retval == FALSE && warn != FALSE)
 		{
@@ -1084,13 +1056,13 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 
 		if (retval == FALSE)
 		{
-			if (err) {
+			if (err)
 				g_error_free (err);
-			}
 			g_free (totem->mrl);
 			totem->mrl = NULL;
-			play_pause_set_label (totem, STATE_STOPPED);
 			bacon_video_widget_set_logo_mode (totem->bvw, TRUE);
+		} else {
+			totem_file_opened (totem, totem->mrl);
 		}
 	}
 	update_buttons (totem);
@@ -1181,10 +1153,36 @@ totem_action_seek_relative (Totem *totem, int off_sec)
 		char *msg, *disp;
 
 		disp = totem_uri_escape_for_display (totem->mrl);
-		msg = g_strdup_printf(_("Totem could not play '%s'."), totem->mrl);
+		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
+		g_free (disp);
+
+		totem_action_stop (totem);
+		totem_action_error (msg, err->message, totem);
+		g_free (msg);
+		g_error_free (err);
+	}
+}
+
+void
+totem_action_seek_time (Totem *totem, gint64 sec)
+{
+	GError *err = NULL;
+
+	if (totem->mrl == NULL)
+		return;
+	if (bacon_video_widget_is_seekable (totem->bvw) == FALSE)
+		return;
+
+	bacon_video_widget_seek_time (totem->bvw, sec, &err);
+
+	if (err != NULL)
+	{
+		char *msg, *disp;
+
+		disp = totem_uri_escape_for_display (totem->mrl);
+		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 		g_free (disp);
 
-		totem_playlist_set_playing (totem->playlist, FALSE);
 		totem_action_stop (totem);
 		totem_action_error (msg, err->message, totem);
 		g_free (msg);
@@ -1366,7 +1364,7 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 			}
 		}
 
-		totem_playlist_add_mrl (totem->playlist, filename, title);
+		totem_playlist_add_mrl (totem->playlist, filename ? filename : p->data, title);
 
 		g_free (filename);
 		g_free (p->data);
@@ -1465,8 +1463,10 @@ on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 	g_free (old_mrl);
 
 	bacon_video_widget_close (totem->bvw);
+	totem_file_closed (totem);
 	totem_gdk_window_set_waiting_cursor (totem->win->window);
 	bacon_video_widget_open (totem->bvw, new_mrl, NULL);
+	totem_file_opened (totem, new_mrl);
 	gdk_window_set_cursor (totem->win->window, NULL);
 	bacon_video_widget_play (bvw, NULL);
 	g_free (new_mrl);
@@ -1492,9 +1492,7 @@ on_channels_change_event (BaconVideoWidget *bvw, Totem *totem)
 	/* updated stream info (new song) */
 	name = totem_get_nice_name_for_stream (totem);
 
-	bacon_video_widget_properties_update
-		(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
-		 totem->bvw);
+	totem_metadata_updated (totem, NULL, NULL, NULL);
 
 	if (name != NULL) {
 		update_mrl_label (totem, name);
@@ -1518,9 +1516,7 @@ on_playlist_change_name (TotemPlaylist *playlist, Totem *totem)
 
 	if (totem_playlist_get_current_metadata (playlist, &artist,
 						 &title, &album) != FALSE) {
-		bacon_video_widget_properties_from_metadata (
-			BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
-			artist, title, album);
+		totem_metadata_updated (totem, artist, title, album);
 
 		g_free (artist);
 		g_free (album);
@@ -1533,9 +1529,7 @@ on_got_metadata_event (BaconVideoWidget *bvw, Totem *totem)
 {
         char *name = NULL;
 
-	bacon_video_widget_properties_update
-		(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
-		 totem->bvw);
+	totem_metadata_updated (totem, NULL, NULL, NULL);
 
 	name = totem_get_nice_name_for_stream (totem);
 
@@ -1570,11 +1564,13 @@ on_buffering_event (BaconVideoWidget *bvw, int percentage, Totem *totem)
 }
 
 static void
-update_seekable (Totem *totem, gboolean seekable)
+update_seekable (Totem *totem)
 {
+	gboolean seekable;
+
+	seekable = bacon_video_widget_is_seekable (totem->bvw);
 	if (totem->seekable == seekable)
 		return;
-
 	totem->seekable = seekable;
 
 	/* Check if the stream is seekable */
@@ -1587,9 +1583,8 @@ update_seekable (Totem *totem, gboolean seekable)
 
 	totem_action_set_sensitivity ("skip-forward", seekable);
 	totem_action_set_sensitivity ("skip-backwards", seekable);
-	totem_action_set_sensitivity ("skip-to", seekable);
-	if (totem->skipto)
-		totem_skipto_set_seekable (totem->skipto, seekable);
+
+	g_object_notify (G_OBJECT (totem), "seekable");
 }
 
 static void
@@ -1599,9 +1594,6 @@ update_current_time (BaconVideoWidget *bvw,
 		float current_position,
 		gboolean seekable, Totem *totem)
 {
-	update_skip_to (totem, stream_length);
-	update_seekable (totem, seekable);
-
 	if (totem->seek_lock == FALSE)
 	{
 		gtk_adjustment_set_value (totem->seekadj,
@@ -1624,9 +1616,11 @@ update_current_time (BaconVideoWidget *bvw,
 		totem_time_label_set_time
 			(TOTEM_TIME_LABEL (totem->tcw_time_label),
 			 current_time, stream_length);
-		bacon_video_widget_properties_from_time
-			(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
-			 stream_length);
+	}
+
+	if (totem->stream_length != stream_length) {
+		g_object_notify (G_OBJECT (totem), "stream-length");
+		totem->stream_length = stream_length;
 	}
 }
 
@@ -1672,15 +1666,23 @@ update_volume_sliders (Totem *totem)
 }
 
 static void
-property_notify_cb (BaconVideoWidget *bvw, GParamSpec *spec, Totem *totem)
+property_notify_cb_volume (BaconVideoWidget *bvw, GParamSpec *spec, Totem *totem)
 {
-	if (strcmp ("volume", spec->name) == 0) {
-		update_volume_sliders (totem);
-	} else if (strcmp ("logo-mode", spec->name) == 0) {
-		gboolean enabled;
-		enabled = bacon_video_widget_get_logo_mode (totem->bvw);
-		totem_action_zoom (totem, enabled ? ZOOM_DISABLE : ZOOM_ENABLE);
-	}
+	update_volume_sliders (totem);
+}
+
+static void
+property_notify_cb_logo_mode (BaconVideoWidget *bvw, GParamSpec *spec, Totem *totem)
+{
+	gboolean enabled;
+	enabled = bacon_video_widget_get_logo_mode (totem->bvw);
+	totem_action_zoom (totem, enabled ? ZOOM_DISABLE : ZOOM_ENABLE);
+}
+
+static void
+property_notify_cb_seekable (BaconVideoWidget *bvw, GParamSpec *spec, Totem *totem)
+{
+	update_seekable (totem);
 }
 
 static gboolean
@@ -1814,6 +1816,8 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 
 		/* Get the subtitle part out for our tests */
 		filename = totem_create_full_path (data);
+		if (filename == NULL)
+			filename = g_strdup (data);
 
 		if (g_file_test (filename, G_FILE_TEST_IS_REGULAR)
 				|| strstr (filename, "#") != NULL
@@ -1821,7 +1825,8 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 				|| g_str_has_prefix (filename, "dvd:") != FALSE
 				|| g_str_has_prefix (filename, "vcd:") != FALSE
 				|| g_str_has_prefix (filename, "cdda:") != FALSE
-				|| g_str_has_prefix (filename, "cd:") != FALSE)
+				|| g_str_has_prefix (filename, "cd:") != FALSE
+				|| g_str_has_prefix (filename, "dvb:") != FALSE)
 		{
 			if (cleared == FALSE)
 			{
@@ -1833,6 +1838,7 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 					 playlist_changed_cb, totem);
 				changed = totem_playlist_clear (totem->playlist);
 				bacon_video_widget_close (totem->bvw);
+				totem_file_closed (totem);
 				cleared = TRUE;
 			}
 
@@ -1842,6 +1848,11 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 			} else if (g_str_has_prefix (filename, "cdda:/") != FALSE) {
 				totem_playlist_add_mrl (totem->playlist, data, NULL);
 				changed = TRUE;
+			} else if (g_str_has_prefix (filename, "dvb:/") != FALSE) {
+				totem_playlist_add_mrl (totem->playlist, data, NULL);
+				changed = TRUE;
+			} else if (g_str_equal (filename, "dvb:") != FALSE) {
+				totem_action_load_media (totem, MEDIA_TYPE_DVB, NULL);
 			} else if (totem_playlist_add_mrl (totem->playlist,
 						filename, NULL) != FALSE) {
 				totem_action_add_recent (totem, filename);
@@ -1863,72 +1874,6 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 	return changed;
 }
 
-static void
-commit_hide_skip_to (GtkDialog *dialog, gint response, Totem *totem)
-{
-	GError *err = NULL;
-
-	if (response != GTK_RESPONSE_OK)
-	{
-		gtk_widget_destroy (GTK_WIDGET (totem->skipto));
-		return;
-	}
-
-	gtk_widget_hide (GTK_WIDGET (dialog));
-
-	bacon_video_widget_seek_time (totem->bvw,
-			totem_skipto_get_range (totem->skipto), &err);
-
-	gtk_widget_destroy (GTK_WIDGET (totem->skipto));
-
-	if (err != NULL)
-	{
-		char *msg, *disp;
-
-		disp = totem_uri_escape_for_display (totem->mrl);
-		msg = g_strdup_printf(_("Totem could not seek in '%s'."), disp);
-		g_free (disp);
-		totem_action_stop (totem);
-		totem_playlist_set_playing (totem->playlist, FALSE);
-		totem_action_error (msg, err->message, totem);
-		g_free (msg);
-		g_error_free (err);
-	}
-}
-
-void
-totem_action_skip_to (Totem *totem)
-{
-	char *filename;
-	TotemSkipto **skipto;
-
-	if (totem->seekable == FALSE)
-		return;
-
-	if (totem->skipto != NULL)
-	{
-		gtk_window_present (GTK_WINDOW (totem->skipto));
-		return;
-	}
-
-	filename = totem_interface_get_full_path ("skip_to.glade");
-	totem->skipto = TOTEM_SKIPTO (totem_skipto_new (filename));
-	g_free (filename);
-
-	g_signal_connect (G_OBJECT (totem->skipto), "delete-event",
-			G_CALLBACK (gtk_widget_destroy), NULL);
-	g_signal_connect (G_OBJECT (totem->skipto), "response",
-			G_CALLBACK (commit_hide_skip_to), totem);
-
-	skipto = &totem->skipto;
-	g_object_add_weak_pointer (G_OBJECT (totem->skipto),
-				   (gpointer *) skipto);
-
-	gtk_window_set_transient_for (GTK_WINDOW (totem->skipto),
-			GTK_WINDOW (totem->win));
-	gtk_widget_show (GTK_WIDGET (totem->skipto));
-}
-
 static void
 on_fs_exit1_activate (GtkButton *button, Totem *totem)
 {
@@ -2043,6 +1988,13 @@ totem_action_toggle_controls (Totem *totem)
  	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), !state);
 }
 
+void
+totem_action_next_angle (Totem *totem)
+{
+	if (totem_playing_dvd (totem->mrl) != FALSE)
+		bacon_video_widget_dvd_event (totem->bvw, BVW_DVD_NEXT_ANGLE);
+}
+
 static void
 on_volume_mute_button (GtkButton *button, Totem *totem)
 {
@@ -2055,7 +2007,7 @@ on_volume_max_button (GtkButton *button, Totem *totem)
 	totem_action_volume_relative (totem, 100);
 }
 
-static void
+void
 totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 {
 	gboolean handled = TRUE;
@@ -2105,11 +2057,16 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		g_assert (url != NULL);
 		totem_playlist_clear (totem->playlist);
 		if (strcmp (url, "dvd:") == 0) {
-			totem_action_play_media (totem, MEDIA_TYPE_DVD);
+			//FIXME b0rked
+			totem_action_play_media (totem, MEDIA_TYPE_DVD, NULL);
 		} else if (strcmp (url, "vcd:") == 0) {
-			totem_action_play_media (totem, MEDIA_TYPE_VCD);
+			//FIXME b0rked
+			totem_action_play_media (totem, MEDIA_TYPE_VCD, NULL);
 		} else if (g_str_has_prefix (url, "cd:") != FALSE) {
-			totem_action_play_media (totem, MEDIA_TYPE_CDDA);
+			//FIXME b0rked
+			totem_action_play_media (totem, MEDIA_TYPE_CDDA, NULL);
+		} else if (g_str_has_prefix (url, "dvb:") != FALSE) {
+			totem_action_play_media (totem, MEDIA_TYPE_DVB, NULL);
 		} else if (g_str_has_prefix (url, "cdda:/") != FALSE) {
 			totem_playlist_add_mrl (totem->playlist, url, NULL);
 		} else if (totem_playlist_add_mrl (totem->playlist,
@@ -2145,6 +2102,20 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 			g_free (title);
 		}
 		break;
+	case TOTEM_REMOTE_COMMAND_SHOW_VOLUME:
+		{
+			char *vol_str;
+			int vol;
+
+			if (bacon_video_widget_can_set_volume (totem->bvw) == FALSE)
+				vol = 0;
+			else
+				vol = bacon_video_widget_get_volume (totem->bvw);
+			vol_str = g_strdup_printf ("%d", vol);
+			bacon_message_connection_send (totem->conn, vol_str);
+			g_free (vol_str);
+		}
+		break;
 	case TOTEM_REMOTE_COMMAND_UP:
 		bacon_video_widget_dvd_event (totem->bvw,
 				BVW_DVD_ROOT_MENU_UP);
@@ -2195,15 +2166,6 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 	}
 }
 
-#ifdef HAVE_REMOTE
-static void
-totem_button_pressed_remote_cb (TotemRemote *remote, TotemRemoteCommand cmd,
-		Totem *totem)
-{
-	totem_action_remote (totem, cmd, NULL);
-}
-#endif /* HAVE_REMOTE */
-
 static void
 playlist_changed_cb (GtkWidget *playlist, Totem *totem)
 {
@@ -2223,7 +2185,8 @@ playlist_changed_cb (GtkWidget *playlist, Totem *totem)
 	{
 		totem_action_set_mrl_and_play (totem, mrl);
 	} else if (totem->mrl != NULL) {
-		totem_playlist_set_playing (totem->playlist, TRUE);
+		//FIXME that's busted
+		totem_playlist_set_playing (totem->playlist, TOTEM_PLAYLIST_STATUS_PLAYING);
 	}
 
 	g_free (mrl);
@@ -2304,9 +2267,33 @@ update_fullscreen_size (Totem *totem)
 gboolean
 totem_is_fullscreen (Totem *totem)
 {
+	g_return_val_if_fail (TOTEM_IS_OBJECT (totem), FALSE);
+
 	return (totem->controls_visibility == TOTEM_CONTROLS_FULLSCREEN);
 }
 
+gboolean
+totem_is_playing (Totem *totem)
+{
+	g_return_val_if_fail (TOTEM_IS_OBJECT (totem), FALSE);
+
+	if (totem->bvw == NULL)
+		return FALSE;
+
+	return bacon_video_widget_is_playing (totem->bvw) != FALSE;
+}
+
+gboolean
+totem_is_seekable (Totem *totem)
+{
+	g_return_val_if_fail (TOTEM_IS_OBJECT (totem), FALSE);
+
+	if (totem->bvw == NULL)
+		return FALSE;
+
+	return bacon_video_widget_is_seekable (totem->bvw) != FALSE;
+}
+
 static void
 move_popups (Totem *totem)
 {
@@ -2485,11 +2472,10 @@ on_eos_event (GtkWidget *widget, Totem *totem)
 		char *mrl;
 
 		/* Set play button status */
-		play_pause_set_label (totem, STATE_PAUSED);
 		totem_playlist_set_at_start (totem->playlist);
 		update_buttons (totem);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
 		totem_action_stop (totem);
+		mrl = totem_playlist_get_current_mrl (totem->playlist);
 		totem_action_set_mrl_with_warning (totem, mrl, FALSE);
 		bacon_video_widget_pause (totem->bvw);
 		g_free (mrl);
@@ -2573,16 +2559,6 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		bacon_video_widget_dvd_event (totem->bvw,
 				BVW_DVD_CHAPTER_MENU);
 		break;
-#ifndef HAVE_GTK_ONLY
-	case GDK_D:
-	case GDK_d:
-		totem_gromit_toggle ();
-		break;
-	case GDK_E:
-	case GDK_e:
-		totem_gromit_clear (FALSE);
-		break;
-#endif /* !HAVE_GTK_ONLY */
 	case GDK_F11:
 	case GDK_f:
 	case GDK_F:
@@ -2590,10 +2566,7 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		break;
 	case GDK_g:
 	case GDK_G:
-		if (totem_playing_dvd (totem->mrl) != FALSE) {
-				bacon_video_widget_dvd_event (totem->bvw,
-					BVW_DVD_NEXT_ANGLE);
-		}
+		totem_action_next_angle (totem);
 		break;
 	case GDK_h:
 	case GDK_H:
@@ -2657,7 +2630,7 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		if (event->state & GDK_CONTROL_MASK) {
 			totem_action_take_screenshot (totem);
 		} else {
-			totem_action_skip_to (totem);
+			return FALSE;
 		}
 		break;
 	case GDK_t:
@@ -2896,18 +2869,6 @@ on_volume_scroll_event (GtkWidget *win, GdkEventScroll *event, Totem *totem)
 	return totem_action_handle_volume_scroll (totem, event->direction);
 }
 
-#ifdef HAVE_MEDIA_PLAYER_KEYS
-static gboolean
-on_window_focus_in_event (GtkWidget *win, GdkEventFocus *event, Totem *totem)
-{
-	if (totem->remote != NULL) {
-		totem_remote_window_activated (totem->remote);
-	}
-
-	return FALSE;
-}
-#endif
-
 static void
 update_media_menu_items (Totem *totem)
 {
@@ -2975,7 +2936,7 @@ totem_setup_window (Totem *totem)
 	GError *err = NULL;
 	GtkWidget *main_pane;
 
-	filename = g_build_filename (g_get_home_dir (), ".gnome2", "totem", NULL);
+	filename = g_build_filename (totem_dot_dir (), "state.ini", NULL);
 	keyfile = g_key_file_new ();
 	if (g_key_file_load_from_file (keyfile, filename,
 			G_KEY_FILE_NONE, NULL) == FALSE) {
@@ -3053,6 +3014,9 @@ totem_callback_connect (Totem *totem)
 {
 	GtkWidget *item, *box, *arrow;
 	GtkAction *action;
+	GtkTooltips *tooltips;
+
+	tooltips = gtk_tooltips_new ();
 
 	/* Menu items */
 	gtk_action_group_set_visible (totem->zoom_action_group,
@@ -3068,22 +3032,34 @@ totem_callback_connect (Totem *totem)
 	/* Controls */
 	box = glade_xml_get_widget (totem->xml, "tmw_buttons_hbox");
 
-	action = gtk_action_group_get_action (totem->main_action_group,
-			"previous-chapter");
-	item = gtk_action_create_tool_item (action);
-	atk_object_set_name (gtk_widget_get_accessible (item),
-			_("Previous Chapter/Movie"));
-	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
-
 	action = gtk_action_group_get_action (totem->main_action_group, "play");
 	item = gtk_action_create_tool_item (action);
 	atk_object_set_name (gtk_widget_get_accessible (item),
 			_("Play / Pause"));
+ 	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (item), 
+ 				   GTK_TOOLTIPS (tooltips),
+ 				   _("Play / Pause"), 
+ 				   NULL);
+	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
+
+	action = gtk_action_group_get_action (totem->main_action_group,
+			"previous-chapter");
+	item = gtk_action_create_tool_item (action);
+	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (item), 
+				   GTK_TOOLTIPS (tooltips),
+				   _("Previous Chapter/Movie"), 
+				   NULL);
+	atk_object_set_name (gtk_widget_get_accessible (item),
+			_("Previous Chapter/Movie"));
 	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
 
 	action = gtk_action_group_get_action (totem->main_action_group,
 			"next-chapter");
 	item = gtk_action_create_tool_item (action);
+	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (item), 
+				   GTK_TOOLTIPS (tooltips),
+				   _("Next Chapter/Movie"), 
+				   NULL);
 	atk_object_set_name (gtk_widget_get_accessible (item),
 			_("Next Chapter/Movie"));
 	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
@@ -3139,13 +3115,13 @@ totem_callback_connect (Totem *totem)
 	/* Control Popup */
 	box = glade_xml_get_widget (totem->xml, "tcw_buttons_hbox");
 
-	action = gtk_action_group_get_action (totem->main_action_group, "previous-chapter");
+	action = gtk_action_group_get_action (totem->main_action_group, "play");
 	item = gtk_action_create_tool_item (action);
 	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
 	g_signal_connect (G_OBJECT (item), "clicked",
 			G_CALLBACK (on_mouse_click_fullscreen), totem);
 
-	action = gtk_action_group_get_action (totem->main_action_group, "play");
+	action = gtk_action_group_get_action (totem->main_action_group, "previous-chapter");
 	item = gtk_action_create_tool_item (action);
 	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
 	g_signal_connect (G_OBJECT (item), "clicked",
@@ -3157,7 +3133,6 @@ totem_callback_connect (Totem *totem)
 	g_signal_connect (G_OBJECT (item), "clicked",
 			G_CALLBACK (on_mouse_click_fullscreen), totem);
 
-
 	item = glade_xml_get_widget (totem->xml, "tcw_volume_mute_button");
 	g_signal_connect (G_OBJECT (item), "clicked",
 			G_CALLBACK (on_volume_mute_button), totem);
@@ -3219,12 +3194,6 @@ totem_callback_connect (Totem *totem)
 	totem_action_set_sensitivity ("previous-chapter", FALSE);
 	totem_action_set_sensitivity ("skip-forward", FALSE);
 	totem_action_set_sensitivity ("skip-backwards", FALSE);
-	totem_action_set_sensitivity ("skip-to", FALSE);
-
-#ifdef HAVE_MEDIA_PLAYER_KEYS
-	g_signal_connect (G_OBJECT(totem->win), "focus-in-event",
-			G_CALLBACK (on_window_focus_in_event), totem);
-#endif
 }
 
 static void
@@ -3268,17 +3237,12 @@ video_widget_create (Totem *totem)
 		{ "text/uri-list", 0, 0 },
 	};
 
-	totem->scr = totem_scrsaver_new ();
-
 	totem->bvw = BACON_VIDEO_WIDGET
 		(bacon_video_widget_new (-1, -1, BVW_USE_TYPE_VIDEO, &err));
 
 	if (totem->bvw == NULL)
 	{
-		totem_playlist_set_playing (totem->playlist, FALSE);
-
 		gtk_widget_hide (totem->win);
-
 		totem_action_error_and_exit (_("Totem could not startup."), err != NULL ? err->message : _("No reason."), totem);
 		if (err != NULL)
 			g_error_free (err);
@@ -3365,8 +3329,12 @@ video_widget_create (Totem *totem)
 	bacon_video_widget_set_volume (totem->bvw,
 			gconf_client_get_int (totem->gc,
 				GCONF_PREFIX"/volume", NULL));
-	g_signal_connect (G_OBJECT (totem->bvw), "notify",
-			G_CALLBACK (property_notify_cb), totem);
+	g_signal_connect (G_OBJECT (totem->bvw), "notify::volume",
+			G_CALLBACK (property_notify_cb_volume), totem);
+	g_signal_connect (G_OBJECT (totem->bvw), "notify::logo-mode",
+			G_CALLBACK (property_notify_cb_logo_mode), totem);
+	g_signal_connect (G_OBJECT (totem->bvw), "notify::seekable",
+			G_CALLBACK (property_notify_cb_seekable), totem);
 	update_volume_sliders (totem);
 }
 
@@ -3469,17 +3437,15 @@ main (int argc, char **argv)
 	gnome_authentication_manager_init ();
 #endif /* !HAVE_GTK_ONLY */
 
-	totem = g_new0 (Totem, 1);
+	totem = g_object_new (TOTEM_TYPE_OBJECT, NULL);
 
 	/* IPC stuff */
 	totem->conn = bacon_message_connection_new (GETTEXT_PACKAGE);
 	if (bacon_message_connection_get_is_server (totem->conn) == FALSE)
 	{
 		totem_options_process_for_server (totem->conn, &optionstate);
-		bacon_message_connection_free (totem->conn);
-		g_free (totem);
 		gdk_notify_startup_complete ();
-		exit (0);
+		totem_action_exit (totem);
 	} else {
 		totem_options_process_early (gc, &optionstate);
 	}
@@ -3502,7 +3468,6 @@ main (int argc, char **argv)
 
 	/* The sidebar */
 	playlist_widget_setup (totem);
-	totem->properties = bacon_video_widget_properties_new ();
 
 	/* The rest of the widgets */
 	totem->state = STATE_STOPPED;
@@ -3523,7 +3488,6 @@ main (int argc, char **argv)
 		(totem->xml, "tcw_volume_hscale");
 	totem->fs_voladj = gtk_range_get_adjustment
 		(GTK_RANGE (totem->fs_volume));
-	totem->tooltip = gtk_tooltips_new ();
 	g_object_set_data (G_OBJECT (totem->fs_volume), "fs", GINT_TO_POINTER (1));
 	totem->volume_first_time = 1;
 	totem->statusbar = glade_xml_get_widget (totem->xml, "tmw_statusbar");
@@ -3560,6 +3524,11 @@ main (int argc, char **argv)
 	/* Command-line handling */
 	totem_options_process_late (totem, &optionstate);
 
+	/* Initialise all the plugins */
+	totem_object_plugins_init (totem);
+	//FIXME we should set the current page again, in case a plugin was
+	// the default page
+
 	if (totem->session_restored != FALSE)
 	{
 		totem_session_restore (totem, optionstate.filenames);
@@ -3577,12 +3546,6 @@ main (int argc, char **argv)
 				totem_message_connection_receive_cb, totem);
 	}
 
-#ifdef HAVE_REMOTE
-	totem->remote = totem_remote_new ();
-	g_signal_connect (totem->remote, "button_pressed",
-			  G_CALLBACK (totem_button_pressed_remote_cb), totem);
-#endif /* HAVE_REMOTE */
-
 	gtk_main ();
 
 #ifndef HAVE_GTK_ONLY
